; ------- QL Screen Taker 1.00 ------
; Inserisce nel QDOS una routine connessa al loop di schedulazione che
; salva sul device voluto lo screen dopo un tempo prefissato.
;
;    +---------------------------------------------------------+
;    | (C) Notarianni Carlo & Mauro x Sud Software MCMLXXXVIII |
;    +---------------------------------------------------------+
;

          lea       procdef(PC),a1  ; linka i nuovi comandi
          move.w    $110,a2         ; e torna al SuperBASIC
          jmp       (a2)            ; senza alcun errore
procdef
          dc.w      5
          dc.w      scrsv_on-*
          dc.b      8,'SCRTK_ON',0
          dc.w      scrsv_off-*
          dc.b      9,'SCRTK_OFF'
          dc.w      scrsv_dev-*
          dc.b      9,'SCRTK_DEV'   ; tabella di definizione
          dc.w      0,4             ; e puntatori alle routine
          dc.w      scrsv_file$-*
          dc.b      11,"SCRTK_FILE$"
          dc.w      scrsv_status-*
          dc.b      12,'SCRTK_STATUS',0
          dc.w      0
;
; Procedura SCRTK_ON <DEVNAME$>,<TIMER> (2 parametri obbligatori)
; inserisce la routine nel loop di schedulazione
; la stringa questa diventa il nome del device dove
; salvare lo screen, TIMER indica dopo quanti secondi la screen
; dev'essere salvato
;
scrsv_on
          moveq     #0,d0
          cmpa.l    a3,a5           ; ci sono parametri ?
          beq       bad_par         ; no: dai un bad_parameter
          movem.l   a3-a5,-(a7)
          addq.l    #8,a3
          move.l    a5,d7
          sub.l     a3,d7
          moveq     #-15,d0
          cmpa.l    a3,a5
          beq       ripristina
          move.w    ca_gtlin,a2     ; preleva interi lunghi
          jsr       (a2)            ;
          bne       ripristina
          cmpi.w    #1,d3           ; piu' di un parametro?
          bgt       ripristina      ; dai BAD_PAR
          moveq     #0,d2
          move.l    0(a6,a1.l),d2   ; metti il valore TIMER in D@
          tst.l     d2
          ble.s     ripristina
          lea       time(pc),a1     ; TIME = D2
          move.l    d2,(a1)         ;
          moveq     #0,d0
ripristina
          movem.l   (a7)+,a3-a5     ;
          tst.l     d0              ; ci sono errori?
          bne       bad_par         ; torna con BAD_PAR
          sub.l     d7,a5
;
          bsr.s     dev2            ; si: preleva prima il nuovo devname
;
          bne.s     bas             ; ritorna con gli eventuali errori
;
          moveq     #mt_rclck,d0
          trap      #1
          lea       oldtime(pc),a1
          move.l    d1,(a1)
;
; inizializza il clock di sistema
;
          moveq     #mt_sclck,d0
          moveq     #0,d1
          trap      #1
;
attiva
          lea       int_link(PC),a0 ; in a0 BUFFER address link
          tst.l     (a0)            ; se <>0 la routine e' gia' attiva
          bne.s     no_err          ; si: torna al BASIC senza errori
;
; La routine, prima disattiva, viene attivata
; in A1 viene passato l'indirizzo d'inizio
; in A0 la base del buffer di lavoro
;
          lea       keyrow(pc),a1   ; a1=address di inizio della routine
          moveq     #mt_lschd,d0    ; d0=manager trap link scheduler
          lea       int_link(PC),a0 ; a0=address d'entrata
          move.l    a1,4(a0)        ; copia l'indirizzo
          trap      #1              ; linka la routine e ...
          bra.s     no_err          ; ritorna senza errori
;
; Procedura SCRTK_OFF <> (NESSUN PARAMETRO)
; se la routine e' attiva la toglie dal
; loop di schedulazione altrimenti ritorna
;

scrsv_off
          cmpa.l    a3,a5           ; ci sono parametri ?
          bne.s     bad_par         ; si: dai bad parameter
          lea       int_link(PC),a0 ; add link in a0 (addr da rimuovere)
          tst.l     (a0)            ; indirizzo=0 ?
          beq.s     no_err          ; si:la routine era gia' disattiva
          moveq     #mt_rschd,d0    ; no:pronto a rimuovere la routine
          trap      #1              ; esegui !
          clr.l    (a0)             ; addr=0 (segnala la disattivazione)
no_err
          moveq     #0,d0           ; nessun errore
bas
          rts                       ; ritorna al BASIC
;
; Procedura SCRTK_DEV <FILENAME$> (1 parametro obbligatorio)
; cambia nome al device su quale verra' salvato lo screen
;

scrsv_dev
          cmpa.l    a3,a5           ; quanti parametri ?
          beq.s     bad_par         ; nessuno: dai BAD PARAMETER !
dev2
          moveq     #$f,d0          ; controlla
          and.b     1(a6,a3.l),d0   ; se e' stata passata
          subq.b    #1,d0           ; una stringa
          bne.s     bad_par         ; no !: BAD PARAMETER
          movea.w   ca_gtstr,a2     ; pronto a prelevare la stringa
          jsr       (a2)            ; chiama il vettore ed esegui !
          subq.w    #1,d3           ; una sola stringa ?
          bne.s     bad_par         ; no ! ....
          add.l     a6,a1           ; a1 punta all'address effettivo
          move.w    (a1)+,d7        ; in d7 la len della stringa
          tst.l     d7              ; len=0 (stringa nulla) ?
          beq.s     bad_par         ; si: bad parameter
          cmpi.w    #34,d7          ; len<=34 ?
          ble.s     okay            ; OKAY !
          move.w    #34,d7          ; len=34 (34=max len di un filename)
okay
          lea       devname(PC),a4  ; a4=address device
copy_string
          move.w    d7,(a4)+        ; passagli la lunghezza
          subq.w    #1,d7           ; sott. 1 per la DBRA
trasf
          move.b    (a1)+,(a4)+     ; copia carattere per carattere
          dbra      d7,trasf        ; per il numero dei caratteri
exit
          bra.s     no_err          ; ritorna con d0=0 (senza errori)
bad_par
          moveq     #-15,d0         ; poni d0=ERR_BP (BAD PARAMETER)
          rts                       ; e ritorna

;
; Funzione SCRTK_FILE$ <> (nessun parametro)
; ritorna al BASIC il devname corrente
;

scrsv_file$
          cmpa.l    a3,a5           ; ci sono parametri ?
          bne.s     bad_par         ; si: dai errore !
          lea       devname(PC),a1  ; a1=address device
          move.w    (a1)+,d7        ; preleva la lunghezza della stringa
          move.l    $58(a6),d0      ; stack in d0
          sub.l     d7,d0           ; fai spazio per la stringa
          subq.l    #2,d0           ; e per la word della len
          bclr      #0,d0           ; rendi pari per forza lindirizzo !
          move.l    d0,$58(a6)      ; nuovo stack aggiornato
          move.l    d0,a4           ; stack in a4 per copiare i caratteri
          add.l     a6,a4           ; a4 punta all'address reale
          moveq     #1,d4           ; tipo parametro=stringa
          bra.s     copy_string     ; copia la stringa e ritorna
;
; Funzione SCRTK_STATUS <> (nessun parametro)
; ritorna il valore del timer se la routine e' attiva,
; altrimenti ritorna 0

scrsv_status
          cmpa.l    a3,a5           ; ci sono parametri ?
          bne.s     bad_par         ; si: allora dai BAD PARAMETER
          move.l    $58(a6),a1      ; a1 punta allo stack aritmetico
          subq.l    #2,a1           ; fai spazio per un intero
          moveq     #0,d1           ; status=off
          lea       int_link(PC),a0 ; add link in a0
          tst.l     (a0)            ; routine e' disattiva ?
          beq.s     rit_funz        ; si: allora ritorna status off
;
          lea       time(pc),a3
          move.l    (a3),d1         ; D1 = numero secondi (status)
rit_funz
          move.w    d1,0(a6,a1.l)   ; status sullo stack (1 o 0)
          move.l    a1,$58(a6)      ; puntatore al dato in bv_rip(a6)
          moveq     #3,d4           ; e' un dato intero (16 bit) !
          bra.s     exit            ; ritorna senza errori

;
;                   *** ROUTINE SCHEDULATA ***
;
; Questa routine, quando e' inserita nel loop di schedulazione, viene
; continuamente richiamata e se stati raggiunti i secondi indicati
; lo screen verra' salvato sul devname indicato nell'apposito buffer.
;

keyrow
          moveq     #mt_rclck,d0
          trap      #1
          lea       time(pc),a1
          move.l    (a1),d2
          cmp.l     d1,d2
          beq.s     screen_save     ; si: allora salva lo screen
          rts                       ; no: fine routine .
screen_save
;
; Sono stati ragiunti i secondi prefissati:
; salva lo screen sul device indicato in  devname
;
          lea       time(pc),a1     ; azzera il timer
          move.l    (a1),d2
          move.l    #0,(a1)         ; (per evitare already exist)
;
          moveq     #mt_sclck,d0
          lea       oldtime(pc),a1
          move.l    (a1),d1
          add.l     d2,d1
          trap      #1
;
          moveq     #mt_ipcom,d0    ; chiama la manager Traps
          lea       suona,a3        ; puntatore al comando IPC
          trap      #1              ; esegui il suono indicato !
;
; Apri un canale sul device in memoria
;
          moveq     #io_open,d0     ; pronto ad aprire un canale
          moveq     #-1,d1          ; per questo JOB
          moveq     #2,d3           ; file nuovo (esclusivo)
          lea       devname,A0      ; indirizzo del nome del canale
          trap      #2              ; esegui la TRAP (apre il canale)
;
; Controlla se ci sono stati errori nel tentativo di aprire il canale
;
          tst.l     d0              ; controlla eventuali errori
          bne.s     segnala_errore  ; errori ? se si allora segnalali
          lea       buff_id(pc),a1  ; a1 punta al buffer del CH ID
          move.l    a0,(a1)         ; salva il CH ID del canale aperto
;
; Salva lo schermo sul device indicato (il canale prima aperto)
; il channel ID si trova in A0 dalla trap precedente

          moveq     #fs_save,d0     ; pronto a salvare un file (SBYTES)
          move.l    #32768,d2       ; lunghezza del file
          move.w    #-1,d3          ; con timeout infinito
          move.l    #131072,a1      ; indirizzo base del file
          trap      #3              ; esegui !
          tst.l     d0              ; controlla eventuali errori
          bne.s     errore2         ; segnala l'errore e chiudi il canale
;
; Chiudi il canale precedentemente aperto bisogna prima
; recuperare il channel id in a0  perso in caso d'errore
;

close_chan
          lea       buff_id(pc),a1  ; a1 punta al buffer del CH ID
          move.l    (a1),a0         ; recupera il CH ID del canale aperto
          moveq     #io_close,d0    ; preparati a chiudere il canale
          trap      #2              ; esegui !!
          rts                       ; ritorna all'inizio del programma
;
; Segnala che vi e' stato un errore (il tipo d'errore e'in D0)
;

segnala_errore
          move.w    ut_err0,a2      ; invia mess. d'errore al canale #0
          jsr       (a2)            ; esegui !
          rts                       ; e ritorna
errore2
          bsr.s     segnala_errore  ; scrivi messaggio d'errore
          bra.s     close_chan      ; e chiudi il canale !

; Inizio area puntatori e buffer di servizio

pointer
          dc.b      9,1,0,0,0,0,7,2 ; IPC pronto a leggere la KEYROW(7)
suona
          dc.b      $A              ; IPC pronto a generare suoni
          DC.B      8               ; i parametri sono 8 (bytes)
          DC.L      $AAAA           ; indica la grandezza
pitch1
          DC.B      22
pitch2
          DC.B      1
speed
          DC.W      0
duration
          DC.W      32760
step_wrap
          DC.B      10
rand_fuzz
          DC.B      10
reply
          DC.B      1               ; no reply


; Buffer dell'indirizzo del link nello scheduler

int_link
          ds.l      2               ; address del LINK nello scheduler

; Serbatoio del nome del devmame

devname
          dc.w      11              ; lunghezza nome
          dc.b      'FLP1_SCREEN',0 ; caratteri del nome
          ds.b      22              ; resto serbatoio nome

; Buffer del CHannel IDentifier del
; canale aperto con il nome in devname

buff_id
          dc.l      1
time
          dc.l      1
oldtime
          dc.l      1

          end


       +---------------------------------------------------------+
       | (C) Notarianni Carlo & Mauro x Sud Software MCMLXXXVIII |
       +---------------------------------------------------------+
